package com.example.versionreturning.domain;

import org.axonframework.commandhandling.CommandHandler;
import org.axonframework.commandhandling.model.AggregateIdentifier;
import org.axonframework.eventsourcing.EventSourcingHandler;
import org.axonframework.eventsourcing.SequenceNumber;
import org.axonframework.spring.stereotype.Aggregate;

import java.util.UUID;

import static org.axonframework.commandhandling.model.AggregateLifecycle.apply;

@Aggregate
public class Counter {

    @AggregateIdentifier
    private UUID id;
    private int count;
    private Long version;

    public Counter() {
    }

    @CommandHandler
    public Counter(CreateCounterCmd cmd) {
        apply(new CounterCreatedEvt(UUID.randomUUID()));
    }

    @CommandHandler
    public Long handle(IncreaseCounterCmd cmd) {
        apply(new CounterIncreasedEvt(id, count + 1));
        return version;
    }

    @EventSourcingHandler
    public void on(CounterCreatedEvt evt, @SequenceNumber long version) {
        id = evt.getId();
        count = 0;
        this.version = version;
    }

    @EventSourcingHandler
    public void on(CounterIncreasedEvt evt, @SequenceNumber long version) {
        count = evt.getNewCount();
        this.version = version;
    }

}
