package com.example.versionreturning.infra;

import org.axonframework.commandhandling.AggregateAnnotationCommandHandler;
import org.axonframework.commandhandling.AnnotationCommandTargetResolver;
import org.axonframework.commandhandling.CommandMessage;
import org.axonframework.commandhandling.VersionedAggregateIdentifier;
import org.axonframework.commandhandling.model.Aggregate;
import org.axonframework.commandhandling.model.Repository;

/**
 * Subclass of AggregateAnnotationCommandHandler that ensures that constructor command handler return not just
 * the aggregate identifier, but a versioned aggregate identifier containing both identifier and version.
 */
public class VersionReturningAggregateAnnotationCommandHandler<T> extends AggregateAnnotationCommandHandler<T> {

    public VersionReturningAggregateAnnotationCommandHandler(Class<T> aggregateType, Repository<T> repository) {
        super(aggregateType, repository, new AnnotationCommandTargetResolver());
    }

    @Override
    protected Object resolveReturnValue(CommandMessage<?> command, Aggregate<T> createdAggregate) {
        return new VersionedAggregateIdentifier(createdAggregate.identifier().toString(), createdAggregate.version());
    }

}
